#ifndef __UART_H
#define __UART_H

#include "types.h"

void init_uart(void);
void putch(char c);
void putstr(char *s);
void putint(unsigned int n);
void puthex(unsigned int n);
bool getch(char *c);

#endif /* __UART_H */
